unit MainForm;

interface

uses
  OpenGL, Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, Textures, glBitmap;

type
  TfrmMain = class(TForm)
    Timer1 : TTimer;
    bStartOpenGL: TButton;
    bStopOpenGL: TButton;
    bDrawGDI: TButton;
    bEnd: TButton;
    procedure FormCreate (Sender : TObject);
    procedure Timer1Timer (Sender : TObject);
    procedure bStartOpenGLClick(Sender: TObject);
    procedure bStopOpenGLClick(Sender: TObject);
    procedure bDrawGDIClick(Sender: TObject);
    procedure bEndClick(Sender: TObject);
  private
    procedure Draw;
    procedure SaveSceenBitmapAsTexture;
  public
  end;

var
  frmMain : TfrmMain;
  Base, Rotation, Changer : GLUInt;
  ToPrint : String;
  gmf : array[0..255] of GLYPHMETRICSFLOAT;
  DC : HDC;
  RC : HGLRC;

  ScreenBitmap : TBitmap;
  ScreenTexture : GLUInt;

  procedure glBindTexture (Target : GLEnum; Texture : GLUInt); stdcall; external OpenGL32;
  procedure glGenTextures(n: GLsizei; var textures: GLuint); stdcall; external opengl32;


implementation


{$R *.DFM}


procedure TfrmMain.SaveSceenBitmapAsTexture;
var
  HBMP : HBitmap;
  ScreenBitmap : TBitmap;
begin
  ScreenBitmap := TBitmap.Create;
  ScreenBitmap.PixelFormat := pf24bit;
  ScreenBitmap.Width := frmMain.ClientWidth;
  ScreenBitmap.Height := frmMain.ClientHeight;
  ScreenBitmap.Canvas.CopyRect (ClientRect,Canvas,ClientRect);

  glGenTextures (1,ScreenTexture);

  GetObject (HBMP,SizeOf (ScreenBitmap),@ScreenBitmap);
  glPixelStorei (GL_UNPACK_ALIGNMENT,4);
  glBindTexture (GL_TEXTURE_2D,ScreenTexture);
  glTexParameteri (GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri (GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
  //gluBuild2DMipmaps (GL_TEXTURE_2D,3,ScreenBitmap.bmWidth,ScreenBitmap.bmHeight,GL_BGR_EXT,GL_UNSIGNED_BYTE,ScreenBitmap.bmBits);
  ScreenBitmap.Handle := HBMP;
  DeleteObject (HBMP);
end;


procedure BuildFont;
var
  Font : HFont;
begin
  base := glGenLists (256);
  Font := CreateFont (-12,-12,0,0,0,0,0,0,ANSI_CHARSET,OUT_TT_PRECIS,CLIP_DEFAULT_PRECIS,ANTIALIASED_QUALITY,FF_DONTCARE or DEFAULT_PITCH,'Arial');
  SelectObject (dc,font);
  wglUseFontOutlines (dc,0,255,base,0,0.2,WGL_FONT_POLYGONS,@gmf);
end;


procedure KillFont;
begin
  glDeleteLists (base,256);
end;


procedure glPrint ( Text : String
                  );
begin
  if (Text <> '') then
  begin
    glTranslatef (-0.22,0,0);
    glPushAttrib (GL_LIST_BIT);
      glListBase (base);
      glCallLists (Length (Text),GL_UNSIGNED_BYTE,PChar (Text));
    glPopAttrib;
  end;
end;


procedure setupPixelFormat ( DC : HDC
                           );
const
   pfd : TPIXELFORMATDESCRIPTOR = (
        nSize : SizeOf (TPIXELFORMATDESCRIPTOR);
        nVersion : 1;
        dwFlags : PFD_SUPPORT_OPENGL or PFD_DRAW_TO_WINDOW or PFD_DOUBLEBUFFER;
        iPixelType : PFD_TYPE_RGBA;
        cColorBits : 24;
        cRedBits : 0;
        cRedShift : 0;
        cGreenBits : 0;
        cGreenShift : 0;
        cBlueBits : 0;
        cBlueShift : 0;
        cAlphaBits : 0;
        cAlphaShift : 0;
        cAccumBits : 0;
        cAccumRedBits : 0;
        cAccumGreenBits : 0;
        cAccumBlueBits : 0;
        cAccumAlphaBits : 0;
        cDepthBits : 16;
        cStencilBits : 0;
        cAuxBuffers : 0;
        iLayerType : PFD_MAIN_PLANE;
        bReserved : 0;
        dwLayerMask : 0;
        dwVisibleMask : 0;
        dwDamageMask : 0;);
var
  PixelFormat : Integer;
begin
   PixelFormat := ChoosePixelFormat (DC,@pfd);
   if (pixelFormat = 0) then
   begin
     Exit;
   end;

   if (SetPixelFormat (DC,PixelFormat,@pfd) <> True) then
   begin
     Exit;
   end;
end;


procedure TfrmMain.FormCreate ( Sender : TObject
                              );
begin
  DC := GetDC (frmMain.Handle);
  SetupPixelFormat (DC);
  RC := wglCreateContext (DC);
  wglMakeCurrent (DC,RC);

  glShadeModel (GL_SMOOTH);
  glClearColor (0,0,0,1);
  glEnable (GL_DEPTH_TEST);
  glHint (GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
  glEnable (GL_LIGHT0);
  glEnable (GL_LIGHTING);
  glEnable (GL_COLOR_MATERIAL);
  BuildFont;

  glViewport (0,0,Width,Height);
  glMatrixMode (GL_PROJECTION);
  glLoadIdentity;
  gluPerspective (45.0,Width/Height,0.1,100.0);
  glMatrixMode (GL_MODELVIEW);
  glLoadIdentity;

  Rotation := 30;
  Changer := 0;
  ToPrint := '3';
  glColor3f (1,0,0);
end;


procedure TfrmMain.Draw;
begin
  glClear (GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

  glLoadIdentity;

  glPushMatrix;
    glEnable (GL_TEXTURE_2D);
    glBindTexture (GL_TEXTURE_2D,ScreenTexture);
    glTranslatef (0,0,-3);
    glBegin (GL_QUADS);
      glTexCoord2f (0,0); glVertex3f (-1.0,-0.9,0);
      glTexCoord2f (1,0); glVertex3f (+1.0,-0.9,0);
      glTexCoord2f (1,1); glVertex3f (+1.0,+0.9,0);
      glTexCoord2f (0,1); glVertex3f (-1.0,+0.9,0);
    glEnd;
    glDisable (GL_TEXTURE_2D);
  glPopMatrix;

  glTranslatef (0,-0.33,-1);
  glRotatef (Rotation,0,1.0,0.0);
  glRotatef (Sin (Changer / 600) * 10,0,1,0);
  glRotatef (Sin (Changer / 600) * 10,1,0,0);
  glPrint (ToPrint);

  SwapBuffers (wglGetCurrentDC);
end;


procedure TfrmMain.Timer1Timer ( Sender : TObject
                               );
begin
  Inc (Changer,5);

  if ((Changer = 360) and (ToPrint = '3')) then
  begin
    glColor3f (1,1,0);
    ToPrint := '2';
    Changer := 0;
  end else
  if ((Changer = 360) and (ToPrint = '2')) then
  begin
    glColor3f (0,1,0);
    ToPrint := '1';
    Changer := 0;
  end else
  if ((Changer = 360) and (ToPrint = '1')) then
  begin
    glColor3f (1,0,0);
    ToPrint := '3';
    Changer := 0;
  end;

  Draw;
end;


procedure TfrmMain.bStartOpenGLClick ( Sender : TObject
                                     );
var
  tgl2DBitmap : TglBitmap2D;
begin
  { Create Bitmap and copy the Form's contents to it. }
  ScreenBitmap := TBitmap.Create;
  ScreenBitmap.PixelFormat := pf24bit;
  ScreenBitmap.Width := frmMain.ClientWidth;
  ScreenBitmap.Height := frmMain.ClientHeight;
  ScreenBitmap.Canvas.CopyRect (ClientRect,Canvas,ClientRect);

  { Convert TBitmap to Texture. }
  tgl2DBitmap := TglBitmap2D.Create;
  tgl2DBitmap.Target := GL_TEXTURE_RECTANGLE_ARB;
  tgl2DBitmap.AssignFromBitmap (ScreenBitmap);
  glGenTextures (1,ScreenTexture);

  { Commence the OpenGL rendering... }
  Timer1.Enabled := True;
end;


procedure TfrmMain.bStopOpenGLClick ( Sender : TObject
                                    );
begin
  Timer1.Enabled := False;
  frmMain.Canvas.Draw (0,0,ScreenBitmap);
end;


procedure TfrmMain.bDrawGDIClick ( Sender : TObject
                                 );
begin
  with frmMain.Canvas do
  begin
    SetBkMode (Handle,Transparent);
    Pen.Color := clRed;
    Ellipse (10,10,100,100);
    Pen.Color := clYellow;
    Rectangle (100,100,200,300);
    Font.Name := 'Arial';
    Font.Size := 20;
    Font.Color := clWhite;
    Font.Style := Font.Style + [fsBold];
    TextOut (50,100,'Here is some text...');
  end;
end;


procedure TfrmMain.bEndClick ( Sender : TObject
                             );
begin
  Halt (0);
end;


end.
